@@ -68,9 +68,9 @@ def create_text_plugin(
         for _ in range(nb_paragraphs):
             max_nb_chars = max_nb_chars or random.randint(200, 400)  # nosec
             paragraphs.append(
-                factory.Faker(
-                    "text", max_nb_chars=max_nb_chars, locale=language
-                ).generate({})
+                factory.Faker("text", max_nb_chars=max_nb_chars).evaluate(
+                    None, None, {"locale": language}
+                )
             )
         body = [container.format(p) for p in paragraphs]
 
@@ -119,7 +119,7 @@ class PermissionFactory(factory.django.DjangoModelFactory):
 class TitleFactory(factory.django.DjangoModelFactory):
     """Create random title objects for CMS pages."""
 
-    language = factory.Iterator([l[0] for l in settings.LANGUAGES])
+    language = factory.Iterator([lang[0] for lang in settings.LANGUAGES])
     page = None
     path = factory.LazyAttribute(lambda o: o.page.get_path_for_slug(o.slug, o.language))
     slug = factory.LazyAttribute(lambda o: slugify(o.title))
@@ -275,7 +275,9 @@ class PageExtensionDjangoModelFactory(factory.django.DjangoModelFactory):
             # Create realistic titles in each language with faker
             languages = languages or [settings.LANGUAGE_CODE]
             title = {
-                language: factory.Faker("catch_phrase", locale=language).generate({})
+                language: factory.Faker("catch_phrase").evaluate(
+                    None, None, {"locale": language}
+                )
                 for language in languages
             }
 
