@@ -3,8 +3,8 @@ from django.core import checks, exceptions, validators
 from django.db import models
 from django.forms import MultipleChoiceField, widgets
 from django.utils.text import capfirst
+from django.utils.translation import gettext_lazy as _
 from django.utils.translation import ngettext_lazy
-from django.utils.translation import ugettext_lazy as _
 
 
 def to_sentence(elements):
@@ -51,7 +51,7 @@ class MultiSelectFormField(MultipleChoiceField):
         Set validators for the field's maximum number of choices.
         """
         self.max_choices = kwargs.pop("max_choices", None)
-        super(MultiSelectFormField, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
         if self.max_choices is not None:
             self.validators.append(MaxChoicesValidator(self.max_choices))
@@ -151,8 +151,8 @@ class MultiSelectField(models.CharField):
 
     def to_python(self, value):
         """Convert a string value to a list value. Used for deserialization and in clean forms."""
-        if isinstance(value, list):
-            return value
+        if not isinstance(value, str) and hasattr(value, "__iter__"):
+            return list(value)
         if not value:
             return None if value is None else []
         return list([v.strip() for v in value.split(",")])
@@ -243,9 +243,7 @@ class MultiSelectField(models.CharField):
 
         We choose to display a sentence of the form: "French, English and German".
         """
-        super(MultiSelectField, self).contribute_to_class(
-            cls, name, private_only=private_only
-        )
+        super().contribute_to_class(cls, name, private_only=private_only)
         if self.choices:
             choicedict = dict(self.choices)
 
