@@ -2,7 +2,7 @@
 Courses CMS plugins
 """
 
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 from cms.plugin_base import CMSPluginBase
 from cms.plugin_pool import plugin_pool
@@ -32,7 +32,7 @@ class OrganizationPlugin(CMSPluginBase):
     fieldsets = ((None, {"fields": ["page", "variant"]}),)
     model = OrganizationPluginModel
     module = PLUGINS_GROUP
-    name = _("Org")
+    name = _("Organization")
     render_template = "courses/plugins/organization.html"
 
     def render(self, context, instance, placeholder):
@@ -41,6 +41,8 @@ class OrganizationPlugin(CMSPluginBase):
                 "instance": instance,
                 "relevant_page": instance.relevant_page,
                 "placeholder": placeholder,
+                "organization_variant": instance.variant
+                or context.get("organization_variant"),
             }
         )
         return context
@@ -55,7 +57,7 @@ class OrganizationsByCategoryPlugin(CMSPluginBase):
     cache = True
     model = OrganizationsByCategoryPluginModel
     module = PLUGINS_GROUP
-    name = _("Org by Category")
+    name = _("Organization by Category")
     render_template = "courses/plugins/organizations_by_category.html"
 
     def render(self, context, instance, placeholder):
@@ -67,6 +69,8 @@ class OrganizationsByCategoryPlugin(CMSPluginBase):
             {
                 "instance": instance,
                 "organizations": instance.relevant_page.category.get_organizations(),
+                "organization_variant": instance.variant
+                or context.get("organization_variant"),
             }
         )
         return context
@@ -79,9 +83,10 @@ class CategoryPlugin(CMSPluginBase):
     """
 
     cache = True
+    fieldsets = ((None, {"fields": ["page", "variant"]}),)
     model = CategoryPluginModel
     module = PLUGINS_GROUP
-    name = _("Cat")
+    name = _("Category")
     render_template = "courses/plugins/category_plugin.html"
 
     def render(self, context, instance, placeholder):
@@ -90,8 +95,10 @@ class CategoryPlugin(CMSPluginBase):
                 "instance": instance,
                 "relevant_page": instance.relevant_page,
                 "placeholder": placeholder,
+                "category_variant": instance.variant or context.get("category_variant"),
             }
         )
+
         return context
 
 
@@ -114,6 +121,7 @@ class CoursePlugin(CMSPluginBase):
                 "instance": instance,
                 "relevant_page": instance.relevant_page,
                 "placeholder": placeholder,
+                "course_variant": instance.variant or context.get("course_variant"),
             }
         )
         return context
@@ -182,7 +190,7 @@ class BlogPostPlugin(CMSPluginBase):
                 "instance": instance,
                 "relevant_page": instance.relevant_page,
                 "placeholder": placeholder,
-                "variant": instance.variant or context.get("variant"),
+                "blogpost_variant": instance.variant or context.get("blogpost_variant"),
             }
         )
         return context
@@ -197,7 +205,7 @@ class ProgramPlugin(CMSPluginBase):
     cache = True
     model = ProgramPluginModel
     module = PLUGINS_GROUP
-    name = _("Prog")
+    name = _("Program")
     render_template = "courses/plugins/program.html"
 
     def render(self, context, instance, placeholder):
