@@ -2,7 +2,7 @@
 Settings specific to the courses application.
 """
 from django.conf import settings
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 ADMIN = "ADMIN"
 
@@ -220,16 +220,45 @@ PAGE_EXTENSION_TOOLBAR_ITEM_POSITION = getattr(
 # Glimpses
 
 BLOGPOST_GLIMPSE_VARIANT_CHOICES = getattr(
-    settings, "RICHIE_BLOGPOST_GLIMPSE_VARIANT_CHOICES", [(None, _("Default"))]
+    settings,
+    "RICHIE_BLOGPOST_GLIMPSE_VARIANT_CHOICES",
+    [
+        (None, _("Inherit")),
+        ("glimpse", _("Default")),
+        ("mini", _("Mini")),
+        ("favorite", _("Favorite")),
+    ],
+)
+CATEGORY_GLIMPSE_VARIANT_CHOICES = getattr(
+    settings,
+    "RICHIE_CATEGORY_GLIMPSE_VARIANT_CHOICES",
+    [
+        (None, _("Inherit")),
+        ("glimpse", _("Default")),
+        ("badge", _("Badge")),
+        ("tag", _("Tag")),
+    ],
 )
 COURSE_GLIMPSE_VARIANT_CHOICES = getattr(
     settings,
     "RICHIE_COURSE_GLIMPSE_VARIANT_CHOICES",
-    [(None, _("Default")), ("small", "small")],
+    [
+        (None, _("Inherit")),
+        ("glimpse", _("Default")),
+        ("glimpse", _("Small")),
+        ("glimpse__large", _("Large")),
+    ],
 )
 
 ORGANIZATION_GLIMPSE_VARIANT_CHOICES = getattr(
-    settings, "RICHIE_ORGANIZATION_GLIMPSE_VARIANT_CHOICES", [(None, _("Default"))]
+    settings,
+    "RICHIE_ORGANIZATION_GLIMPSE_VARIANT_CHOICES",
+    [
+        (None, _("Inherit")),
+        ("glimpse", _("Default")),
+        ("card", _("Card")),
+        ("row", _("Row")),
+    ],
 )
 
 # For each type of page we define:
@@ -243,7 +272,6 @@ CATEGORIES_PAGE = {
     "reverse_id": "categories",
     "template": "courses/cms/category_detail.html",
 }
-COURSERUNS_PAGE = {"template": "courses/cms/course_run_detail.html"}
 COURSES_PAGE = {"reverse_id": "courses", "template": "courses/cms/course_detail.html"}
 ORGANIZATIONS_PAGE = {
     "reverse_id": "organizations",
