@@ -6,11 +6,14 @@ from collections import namedtuple
 from datetime import datetime, timedelta
 
 from django.utils import timezone
+from django.utils.translation import gettext_lazy as _
 
 import factory
 import pytz
 from cms.api import add_plugin
 
+from richie.plugins.nesteditem.defaults import ACCORDION
+
 from ..core.defaults import ALL_LANGUAGES
 from ..core.factories import (
     FilerImageFactory,
@@ -199,6 +202,8 @@ class CourseFactory(PageExtensionDjangoModelFactory):
     # fields concerning the related page
     page_template = models.Course.PAGE["template"]
 
+    code = factory.Sequence(lambda k: f"{k:05d}")
+
     # pylint: disable=no-self-use
     @factory.lazy_attribute
     def duration(self):
@@ -332,6 +337,38 @@ class CourseFactory(PageExtensionDjangoModelFactory):
                         **{"licence": licence},
                     )
 
+    @factory.post_generation
+    # pylint: disable=unused-argument
+    def fill_plan(self, create, extracted, **kwargs):
+        """Add a course plan composed of random nested items."""
+        if create and extracted:
+            for language in self.extended_object.get_languages():
+                placeholder = self.extended_object.placeholders.get(slot="course_plan")
+                container = add_plugin(
+                    language=language,
+                    placeholder=placeholder,
+                    plugin_type="NestedItemPlugin",
+                    variant=ACCORDION,
+                )
+                for chapter in range(random.randint(2, 4)):  # nosec
+                    chapter_plugin = add_plugin(
+                        language=language,
+                        placeholder=placeholder,
+                        plugin_type="NestedItemPlugin",
+                        target=container,
+                        content=f"Chapter {chapter:d}",
+                        variant=ACCORDION,
+                    )
+                    for part in range(random.randint(2, 4)):  # nosec
+                        add_plugin(
+                            language=language,
+                            placeholder=placeholder,
+                            plugin_type="NestedItemPlugin",
+                            target=chapter_plugin,
+                            content=f"Part {part:d}",
+                            variant=ACCORDION,
+                        )
+
     @factory.post_generation
     # pylint: disable=unused-argument
     def fill_texts(self, create, extracted, **kwargs):
@@ -342,16 +379,18 @@ class CourseFactory(PageExtensionDjangoModelFactory):
         in a list of slot names.
         """
         if create and extracted:
-            for slot in extracted:
+            for slot, plugin_type in extracted.items():
                 create_text_plugin(
                     self.extended_object,
                     slot,
                     nb_paragraphs=1,
                     languages=self.extended_object.get_languages(),
+                    is_html=plugin_type != "PlainTextPlugin",
+                    plugin_type=plugin_type,
                 )
 
 
-class CourseRunFactory(PageExtensionDjangoModelFactory):
+class CourseRunFactory(factory.django.DjangoModelFactory):
     """
     A factory to automatically generate random yet meaningful course runs in our tests.
 
@@ -363,45 +402,22 @@ class CourseRunFactory(PageExtensionDjangoModelFactory):
 
     class Meta:
         model = models.CourseRun
-        exclude = [
-            "page_in_navigation",
-            "page_languages",
-            "page_parent",
-            "page_template",
-            "page_title",
-        ]
-
-    # fields concerning the related page
-    page_template = models.CourseRun.PAGE["template"]
-    page_title = factory.Sequence("session {:d}".format)
 
+    direct_course = factory.SubFactory(CourseFactory)
+    title = factory.Sequence(_("Run {:d}").format)
     resource_link = factory.Faker("uri")
+    sync_mode = models.CourseRunSyncMode.SYNC_TO_PUBLIC
 
-    @factory.lazy_attribute
-    def page_languages(self):
-        """
-        Try getting the list of languages from the parent page and default to None.
-        """
-        return (
-            self.page_parent.get_languages()
-            if getattr(self, "page_parent", None)
-            else None
-        )
-
+    # pylint: disable=no-self-use
     @factory.lazy_attribute
     def languages(self):
         """
-        Try getting the list of languages from the parent page and default to a random set of
-        languages from the complete list of Django supported languages.
+        Compute a random set of languages from the complete list of Django supported languages.
         """
-        return (
-            self.parent.get_languages()
-            if getattr(self, "parent", None)
-            else [
-                l[0]
-                for l in random.sample(ALL_LANGUAGES, random.randint(1, 5))  # nosec
-            ]
-        )
+        return [
+            language[0]
+            for language in random.sample(ALL_LANGUAGES, random.randint(1, 5))  # nosec
+        ]
 
     # pylint: disable=no-self-use
     @factory.lazy_attribute
@@ -639,10 +655,8 @@ class BlogPostFactory(PageExtensionDjangoModelFactory):
 
             for language in self.extended_object.get_languages():
                 text = factory.Faker(
-                    "text",
-                    max_nb_chars=random.randint(50, 100),  # nosec
-                    locale=language,
-                ).generate({})
+                    "text", max_nb_chars=random.randint(50, 100)  # nosec
+                ).evaluate(None, None, {"locale": language})
                 add_plugin(
                     language=language,
                     placeholder=placeholder,
@@ -735,6 +749,22 @@ class PersonFactory(PageExtensionDjangoModelFactory):
                 "bio",
                 nb_paragraphs=1,
                 languages=self.extended_object.get_languages(),
+                is_html=False,
+                plugin_type="PlainTextPlugin",
+            )
+
+    @factory.post_generation
+    # pylint: disable=unused-argument
+    def fill_maincontent(self, create, extracted, **kwargs):
+        """
+        Add a text plugin for maincontent with a long random text
+        """
+        if create and extracted:
+            create_text_plugin(
+                self.extended_object,
+                "maincontent",
+                nb_paragraphs=1,
+                languages=self.extended_object.get_languages(),
             )
 
     @factory.post_generation
@@ -831,10 +861,8 @@ class ProgramFactory(PageExtensionDjangoModelFactory):
 
             for language in self.extended_object.get_languages():
                 text = factory.Faker(
-                    "text",
-                    max_nb_chars=random.randint(50, 100),  # nosec
-                    locale=language,
-                ).generate({})
+                    "text", max_nb_chars=random.randint(50, 100)  # nosec
+                ).evaluate(None, None, {"locale": language})
                 add_plugin(
                     language=language,
                     placeholder=placeholder,
