@@ -6,7 +6,7 @@ from django.conf import settings
 from django.db import models
 from django.db.models import Prefetch
 from django.utils import translation
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 from cms.api import Page
 from cms.extensions.extension_pool import extension_pool
@@ -14,7 +14,7 @@ from cms.models import Title
 from cms.models.pluginmodel import CMSPlugin
 
 from ...core.models import BasePageExtension, PagePluginMixin
-from ..defaults import CATEGORIES_PAGE
+from ..defaults import CATEGORIES_PAGE, CATEGORY_GLIMPSE_VARIANT_CHOICES
 
 
 class Category(BasePageExtension):
@@ -34,6 +34,7 @@ class Category(BasePageExtension):
         db_table = "richie_category"
         ordering = ["-pk"]
         verbose_name = _("category")
+        verbose_name_plural = _("categories")
 
     def __str__(self):
         """Human representation of a category"""
@@ -47,6 +48,17 @@ class Category(BasePageExtension):
             title=self.extended_object.get_title(),
         )
 
+    def get_es_id(self):
+        """
+        An ID built with the node path and the position of this category in the taxonomy:
+            - P: parent, the category has children
+            - L: leaf, the category has no children
+
+        For example: a parent category `P_00010002` and its child `L_000100020001`.
+        """
+        page = self.extended_object
+        return f"{'L' if page.node.is_leaf() else 'P':s}-{page.node.path:s}"
+
     def get_meta_category(self):
         """
         Get the meta category the current category falls under. Meta-categories are a special kind
@@ -199,10 +211,19 @@ class CategoryPluginModel(PagePluginMixin, CMSPlugin):
         limit_choices_to=get_category_limit_choices_to,
         on_delete=models.CASCADE,
     )
+    variant = models.CharField(
+        _("variant"),
+        max_length=50,
+        choices=CATEGORY_GLIMPSE_VARIANT_CHOICES,
+        help_text=_("Optional glimpse variant for a custom look."),
+        blank=True,
+        null=True,
+    )
 
     class Meta:
         db_table = "richie_category_plugin"
         verbose_name = _("category plugin")
+        verbose_name_plural = _("category plugins")
 
 
 extension_pool.register(Category)
