@@ -7,14 +7,16 @@ from datetime import MAXYEAR, datetime
 from django.conf import settings
 from django.core.exceptions import ValidationError
 from django.db import models
-from django.db.models import Q
+from django.db.models import Prefetch, Q
+from django.urls import reverse
 from django.utils import timezone, translation
 from django.utils.functional import lazy
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 import pytz
+from cms.constants import PUBLISHER_STATE_DIRTY
 from cms.extensions.extension_pool import extension_pool
-from cms.models import Page, PagePermission
+from cms.models import Page, PagePermission, Title
 from cms.models.pluginmodel import CMSPlugin
 from filer.fields.image import FilerImageField
 from filer.models import FolderPermission
@@ -30,6 +32,7 @@ from .. import defaults
 from .category import Category
 from .organization import Organization
 from .person import Person
+from .program import Program
 from .role import PageRole
 
 MAX_DATE = datetime(MAXYEAR, 12, 31, tzinfo=pytz.utc)
@@ -38,25 +41,35 @@ MAX_DATE = datetime(MAXYEAR, 12, 31, tzinfo=pytz.utc)
 class CourseState(Mapping):
     """An immutable object to describe a course (resp. course run) state."""
 
-    STATE_CALLS_TO_ACTION = (
-        _("enroll now"),
-        _("enroll now"),
-        None,
-        None,
-        None,
-        None,
-        None,
-    )
+    (
+        ONGOING_OPEN,
+        FUTURE_OPEN,
+        FUTURE_NOT_YET_OPEN,
+        FUTURE_CLOSED,
+        ONGOING_CLOSED,
+        ARCHIVED,
+        TO_BE_SCHEDULED,
+    ) = range(7)
 
-    STATE_TEXTS = (
-        _("closing on"),
-        _("starting on"),
-        _("starting on"),
-        _("enrollment closed"),
-        _("on-going"),
-        _("archived"),
-        _("to be scheduled"),
-    )
+    STATE_CALLS_TO_ACTION = {
+        ONGOING_OPEN: _("enroll now"),
+        FUTURE_OPEN: _("enroll now"),
+        FUTURE_NOT_YET_OPEN: None,
+        FUTURE_CLOSED: None,
+        ONGOING_CLOSED: None,
+        ARCHIVED: None,
+        TO_BE_SCHEDULED: None,
+    }
+
+    STATE_TEXTS = {
+        ONGOING_OPEN: _("closing on"),
+        FUTURE_OPEN: _("starting on"),
+        FUTURE_NOT_YET_OPEN: _("starting on"),
+        FUTURE_CLOSED: _("enrollment closed"),
+        ONGOING_CLOSED: _("on-going"),
+        ARCHIVED: _("archived"),
+        TO_BE_SCHEDULED: _("to be scheduled"),
+    }
 
     def __init__(self, priority, date_time=None):
         """
@@ -74,14 +87,18 @@ class CourseState(Mapping):
           6: theres's no run with a start date or no run at all > "to be scheduled": {None}
         """
         # Check that `date_time` is set when it should be
-        if date_time is None and priority in [0, 1, 2]:
+        if date_time is None and priority in [
+            CourseState.ONGOING_OPEN,
+            CourseState.FUTURE_OPEN,
+            CourseState.FUTURE_NOT_YET_OPEN,
+        ]:
             raise ValidationError(
                 "date_time should not be null for a {:d} course state.".format(priority)
             )
 
         # A special case of being open is when enrollment never ends
         text = self.STATE_TEXTS[priority]
-        if priority == 0 and date_time.year == MAXYEAR:
+        if priority == CourseState.ONGOING_OPEN and date_time.year == MAXYEAR:
             text = _("forever open")
             date_time = None
         kwargs = {
@@ -118,6 +135,15 @@ class Course(BasePageExtension):
     page that presents the course.
     """
 
+    CODE_MAX_LENGTH = 100
+
+    code = models.CharField(
+        _("code"),
+        max_length=CODE_MAX_LENGTH,
+        help_text=_("Unique reference for the course."),
+        null=True,
+        blank=True,
+    )
     effort = EffortField(
         time_units=defaults.TIME_UNITS,
         default_effort_unit=defaults.DEFAULT_EFFORT_UNIT,
@@ -133,12 +159,18 @@ class Course(BasePageExtension):
         blank=True,
         null=True,
     )
+    is_listed = models.BooleanField(
+        default=True,
+        verbose_name=_("is listed"),
+        help_text=_("Tick if you want the course to be visible on the search page."),
+    )
 
     PAGE = defaults.COURSES_PAGE
 
     class Meta:
         db_table = "richie_course"
         verbose_name = _("course")
+        verbose_name_plural = _("courses")
 
     def __str__(self):
         """Human representation of a course."""
@@ -147,6 +179,47 @@ class Course(BasePageExtension):
             title=self.extended_object.get_title(),
         )
 
+    def validate_unique(self, exclude=None):
+        """
+        We can't enforce code unicity with a db constraint because it is repeated across
+        the draft and public versions of the page AND in each snapshot of a course.
+
+        A complex unique constraint would have to span the "extended_object" public key
+        which is not possible.
+        """
+        super().validate_unique()
+        if (
+            self.extended_object.publisher_is_draft
+            and not self.is_snapshot
+            and self.__class__.objects.exclude(pk=self.pk)
+            .filter(
+                code=self.code,
+                # allow public duplicates
+                extended_object__publisher_is_draft=True,
+                # allow snapshot duplicates
+                extended_object__node__parent__cms_pages__course__isnull=True,
+            )
+            .exists()
+        ):
+            raise ValidationError({"code": ["Code must be unique."]})
+
+    def get_admin_url_to_add_run(self, request):
+        """
+        Get the admin url of the form to add a course run with the "direct_course"
+        field pre-populated to self.
+        """
+        base_url = reverse("admin:courses_courserun_add")
+        return f"{base_url:s}?direct_course={self.id:d}"
+
+    @property
+    def is_snapshot(self):
+        """Return True if the course is a snapshot (it has a course as parent)."""
+        # Look for the page and not the course as the course may not yet be created in database
+        # (see "validate_unique" method below)
+        return Page.objects.filter(
+            id=self.extended_object.id, node__parent__cms_pages__course__isnull=False
+        ).exists()
+
     def create_page_role(self):
         """
         Create a new page role for the course with:
@@ -234,7 +307,6 @@ class Course(BasePageExtension):
         language = language or translation.get_language()
 
         selector = "extended_object__person_plugins__cmsplugin_ptr"
-        # pylint: disable=no-member
         filter_dict = {
             "{:s}__language".format(selector): language,
             "{:s}__placeholder__page".format(selector): self.extended_object,
@@ -346,73 +418,95 @@ class Course(BasePageExtension):
             )
         return page_query.distinct()
 
-    def get_course_runs(self):
+    def get_snapshots(self, include_self=False):
         """
-        Returns a query yielding the course runs related to the course. They may be direct
-        children of the course or children of a snapshot of the course.
+        Get a query yielding all snapshots of a course.
 
-        For a draft course instance, the related draft course runs are retrieved.
-        For a public course instance, the related public course runs are retrieved, but only
-        those that are currently published in at least one language*.
+        Arguments:
+        ----------
+        include_self (boolean): whether the current object should be included in the query
+            or just its snapshots.
+
+        Returns:
+        --------
+        Queryset[Course]: All snapshots of a course (calling this method on a snapshot would
+            return an empty queryset)
 
-        (*) The catch here is that a course run could have been published and then unpublished
-            for all languages. The public instance is created the first time a draft page is
-            published. Of course, this public instance will still exist if the object is then
-            unpublished for all languages...
-            Said differently, a page can have a public version of itself in database but not be
-            currently published in any language.
         """
-        node = self.extended_object.node
         is_draft = self.extended_object.publisher_is_draft
-        filter_dict = {
-            "extended_object__publisher_is_draft": is_draft,
-            "extended_object__node__path__startswith": node.path,
-            "extended_object__node__depth__gt": node.depth,
-        }
-        # For a public course, we must filter out course runs that are not published in
-        # any language
-        if is_draft is False:
-            filter_dict["extended_object__title_set__published"] = True
+        node = self.extended_object.node
+        current_and_descendant_nodes = node.__class__.get_tree(parent=node)
 
-        return (
-            CourseRun.objects.filter(**filter_dict)
-            .order_by("extended_object__node__path")
-            .distinct()
+        query = self.__class__.objects.filter(
+            extended_object__node__in=current_and_descendant_nodes,
+            extended_object__publisher_is_draft=is_draft,
         )
 
-    def get_course_runs_for_language(self, language=None):
-        """
-        Returns a query yielding the course runs related to the course for the current
-        language (or the language passed in arguments). They may be direct children of
-        the course or children of a snapshot of the course.
-        """
-        language = language or translation.get_language()
-        course_runs = self.get_course_runs()
+        if include_self is False:
+            query = query.exclude(pk=self.pk)
 
-        # For a public course, we must filter out course runs that are not published in
-        # the language
-        if self.extended_object.publisher_is_draft is False:
-            course_runs = course_runs.filter(
-                extended_object__title_set__language=language,
-                extended_object__title_set__published=True,
-            )
-        else:
-            course_runs = course_runs.filter(
-                extended_object__title_set__language=language
-            )
+        return query
 
-        return course_runs
+    def get_course_runs(self):
+        """
+        Returns a query yielding the course runs related to the course. They may be directly
+        related to the course or to a snapshot of the course.
+
+        The draft and the public page have their own course runs. The course runs on the draft
+        page are copied to the public page when the course is published (for any language).
+        """
+        is_draft = self.extended_object.publisher_is_draft
+        node = self.extended_object.node
+        current_and_descendant_nodes = node.__class__.get_tree(parent=node)
+
+        return CourseRun.objects.filter(
+            direct_course__extended_object__node__in=current_and_descendant_nodes,
+            direct_course__extended_object__publisher_is_draft=is_draft,
+        ).order_by("-start")
 
     def get_course_runs_dict(self):
         """Returns a dict of course runs grouped by their state."""
         course_runs_dict = {
             i: [] for i in range(len(CourseState.STATE_CALLS_TO_ACTION))
         }
-        for run in self.get_course_runs_for_language():
+        for run in self.get_course_runs():
             course_runs_dict[run.state["priority"]].append(run)
 
         return dict(course_runs_dict)
 
+    def get_programs(self, language=None):
+        """
+        Return a query to get the programs related to this course ie for which a plugin for
+        this course is linked to the program page via any placeholder.
+        """
+        is_draft = self.extended_object.publisher_is_draft
+        course = self if is_draft else self.draft_extension
+        language = language or translation.get_language()
+
+        bfs = (
+            "extended_object__placeholders__cmsplugin__courses_coursepluginmodel__page"
+        )
+        filter_dict = {
+            "extended_object__publisher_is_draft": is_draft,
+            "extended_object__placeholders__cmsplugin__language": language,
+            bfs: course.extended_object,
+        }
+
+        # pylint: disable=no-member
+        return (
+            Program.objects.filter(**filter_dict)
+            .select_related("extended_object")
+            .prefetch_related(
+                Prefetch(
+                    "extended_object__title_set",
+                    to_attr="prefetched_titles",
+                    queryset=Title.objects.filter(language=language),
+                )
+            )
+            .order_by("extended_object__node__path")
+            .distinct()
+        )
+
     @property
     def state(self):
         """
@@ -421,20 +515,57 @@ class Course(BasePageExtension):
         The game is to find the highest priority state for this course among its course runs.
         """
         # The default state is for a course that has no course runs
-        best_state = CourseState(6)
+        best_state = CourseState(CourseState.TO_BE_SCHEDULED)
 
-        for course_run in self.get_course_runs_for_language().only(
+        for course_run in self.get_course_runs().only(
             "start", "end", "enrollment_start", "enrollment_end"
         ):
             state = course_run.state
             if state < best_state:
                 best_state = state
-            if state == 0:
+            if state["priority"] == CourseState.ONGOING_OPEN:
                 # We found the best state, don't waste more time
                 break
 
         return best_state
 
+    def copy_relations(self, oldinstance, language=None):
+        """
+        This method is called for 2 types of copying:
+            1- cloning (copying a draft page to another position in the page tree)
+            2- publishing (copying a draft page to its public counterpart)
+
+        When cloning a course, we do not want the related course runs to follow. Each copy of the
+        course has its own distinct runs.
+
+        When publishing a course, the course runs must be manually copied to the public page as
+        it should perfectly reflect the state of the draft page at the moment we clicked on the
+        publish button.
+        """
+        # Don't copy the course runs if this is a copy to clone
+        if oldinstance.public_extension_id != self.id:
+            return
+
+        # Since this is a copy to publish, let's copy the course runs
+        for old_course_run in oldinstance.runs.all():
+            course_run = CourseRun.objects.get(pk=old_course_run.pk)
+            course_run.direct_course = self
+
+            # If the course run already exists on the public course, update it,
+            try:
+                public_course_run = old_course_run.public_course_run
+            except CourseRun.DoesNotExist:
+                # Otherwise create a new public course run and link it to its draft
+                course_run.draft_course_run_id = old_course_run.id
+                course_run.pk = None
+                course_run.save()
+            else:
+                course_run.draft_course_run_id = old_course_run.id
+                course_run.pk = public_course_run.pk
+                course_run.save()
+
+            course_run.copy_translations(old_course_run, language=language)
+
     def save(self, *args, **kwargs):
         """
         Enforce validation each time an instance is saved
@@ -443,13 +574,43 @@ class Course(BasePageExtension):
         super().save(*args, **kwargs)
 
 
-class CourseRun(BasePageExtension):
+class CourseRunSyncMode(models.TextChoices):
+    """Course run synchronization mode choices for the "sync_mode" field."""
+
+    MANUAL = "manual", _("Manual")
+    SYNC_TO_DRAFT = "sync_to_draft", _("Synchronization to draft page")
+    SYNC_TO_PUBLIC = "sync_to_public", _("Synchronization to public page")
+
+
+class CourseRun(TranslatableModel):
     """
     The course run represents and records the occurence of a course between a start
     and an end date.
     """
 
-    resource_link = models.URLField(_("Resource link"), blank=True, null=True)
+    direct_course = models.ForeignKey(
+        Course, on_delete=models.CASCADE, related_name="runs"
+    )
+    # We register the foreign key in "draft_course_run" and not in "public_course_run"
+    # so that the public course run gets deleted by cascade in the database when the
+    # draft page is deleted. Doing it the other way would be fragile.
+    draft_course_run = models.OneToOneField(
+        "self",
+        on_delete=models.CASCADE,
+        null=True,
+        editable=False,
+        related_name="public_course_run",
+    )
+    sync_mode = models.CharField(
+        max_length=20,
+        choices=CourseRunSyncMode.choices,
+        default=CourseRunSyncMode.MANUAL,
+    )
+
+    title = TranslatedField()
+    resource_link = models.CharField(
+        _("resource link"), max_length=200, blank=True, null=True
+    )
     start = models.DateTimeField(_("course start"), blank=True, null=True)
     end = models.DateTimeField(_("course end"), blank=True, null=True)
     enrollment_start = models.DateTimeField(
@@ -466,27 +627,104 @@ class CourseRun(BasePageExtension):
         help_text=_("The list of languages in which the course content is available."),
     )
 
-    PAGE = defaults.COURSERUNS_PAGE
-
     class Meta:
         db_table = "richie_course_run"
         verbose_name = _("course run")
+        verbose_name_plural = _("course runs")
 
     def __str__(self):
         """Human representation of a course run."""
         start = "{:%y/%m/%d %H:%M} - ".format(self.start) if self.start else ""
-        return "{start:s}{course:s}".format(
-            course=self.extended_object.get_title(), start=start
+        return f"Course run {self.id!s} starting {start:s}"
+
+    def copy_translations(self, oldinstance, language=None):
+        """Copy translation objects for a language if provided or for all languages."""
+        query = CourseRunTranslation.objects.filter(master=oldinstance)
+        if language:
+            query = query.filter(language_code=language)
+
+        for translation_object in query:
+            try:
+                target_pk = CourseRunTranslation.objects.filter(
+                    master=self, language_code=translation_object.language_code
+                ).values_list("pk", flat=True)[0]
+            except IndexError:
+                translation_object.pk = None
+            else:
+                translation_object.pk = target_pk
+            translation_object.master = self
+            translation_object.save()
+
+    def mark_course_dirty(self):
+        """
+        Mark the related course page as dirty if the course run has changed since it was last
+        published, so that the modifications can be checked and confirmed by a reviewer.
+        """
+        try:
+            public_instance = self.__class__.objects.get(draft_course_run__pk=self.pk)
+        except self.__class__.DoesNotExist:
+            # This is a new instance, mark page dirty in all languages unless
+            # the course run is yet to be scheduled (hidden from public page in this case)
+            if self.state["priority"] < CourseState.TO_BE_SCHEDULED:
+                self.direct_course.extended_object.title_set.update(
+                    publisher_state=PUBLISHER_STATE_DIRTY
+                )
+            return
+
+        is_visible = (
+            self.state["priority"] < CourseState.TO_BE_SCHEDULED
+            or public_instance.state["priority"] < CourseState.TO_BE_SCHEDULED
         )
+        # Mark the related course page dirty if the course run content has changed
+        # Break out of the for loop as soon as we found a difference
+        for field in self._meta.fields:
+            if field.name == "direct_course":
+                if (
+                    public_instance.direct_course.draft_extension != self.direct_course
+                    and is_visible
+                ):
+                    self.direct_course.extended_object.title_set.update(
+                        publisher_state=PUBLISHER_STATE_DIRTY
+                    )  # mark target page dirty in all languages
+                    page = public_instance.direct_course.draft_extension.extended_object
+                    page.title_set.update(
+                        publisher_state=PUBLISHER_STATE_DIRTY
+                    )  # mark source page dirty in all languages
+                    break
+            elif (
+                field.editable
+                and not field.auto_created
+                and getattr(public_instance, field.name) != getattr(self, field.name)
+                and is_visible
+            ):
+                self.direct_course.extended_object.title_set.update(
+                    publisher_state=PUBLISHER_STATE_DIRTY
+                )  # mark page dirty in all languages
+                break
 
-    # pylint: disable=arguments-differ
     def save(self, *args, **kwargs):
-        """
-        Enforce validation each time an instance is saved.
-        """
+        """Enforce validation each time an instance is saved."""
         self.full_clean()
         super().save(*args, **kwargs)
 
+    # pylint: disable=signature-differs
+    def delete(self, *args, **kwargs):
+        """
+        Mark the related course page as dirty if the course about to be deleted was
+        published and visible (not to be scheduled).
+        """
+        try:
+            # pylint: disable=no-member
+            public_course_run = self.public_course_run
+        except CourseRun.DoesNotExist:
+            pass
+        else:
+            if public_course_run.state["priority"] < CourseState.TO_BE_SCHEDULED:
+                self.direct_course.extended_object.title_set.update(
+                    publisher_state=PUBLISHER_STATE_DIRTY
+                )  # mark page dirty in all languages
+        return super().delete(*args, **kwargs)
+
     # pylint: disable=too-many-return-statements
     @staticmethod
     def compute_state(start, end, enrollment_start, enrollment_end):
@@ -532,23 +770,69 @@ class CourseRun(BasePageExtension):
 
     def get_course(self):
         """Get the course for this course run."""
-        nodes = self.extended_object.node.get_ancestors()
-        return Course.objects.get(
+        is_draft = self.direct_course.extended_object.publisher_is_draft
+        ancestor_nodes = self.direct_course.extended_object.node.get_ancestors()
+        return Course.objects.filter(
             # Joining on `cms_pages` generate duplicates for courses that are under a parent page
             # when this page exists both in draft and public versions. We need to exclude the
             # parent public page to avoid this duplication
             Q(
-                extended_object__node__parent__cms_pages__publisher_is_draft=True
+                extended_object__node__cms_pages__publisher_is_draft=is_draft
             )  # course has a parent
-            | Q(extended_object__node__parent__isnull=True),  # course has no parent
-            # Target courses that are ancestors of the course run
-            extended_object__node__in=nodes,
+            | Q(extended_object__node__isnull=True),  # course has no parent
+            # Target courses that are ancestors of the course related to the course run
+            Q(id=self.direct_course_id) | Q(extended_object__node__in=ancestor_nodes),
             # Exclude snapshots
             extended_object__node__parent__cms_pages__course__isnull=True,  # exclude snapshots
             # Get the course in the same version as the course run
-            extended_object__publisher_is_draft=self.extended_object.publisher_is_draft,
+            extended_object__publisher_is_draft=is_draft,
+        ).distinct()[0]
+
+
+class CourseRunTranslation(TranslatedFieldsModel):
+    """
+    CourseRun Translation model.
+
+    Django parler model linked to the CourseRun to internationalize the fields.
+    """
+
+    master = models.ForeignKey(CourseRun, models.CASCADE, related_name="translations")
+    title = models.CharField(_("title"), null=True, blank=True, max_length=255)
+
+    class Meta:
+        db_table = "richie_course_run_translation"
+        unique_together = ("language_code", "master")
+        verbose_name = _("Course run translation")
+        verbose_name_plural = _("Course run translations")
+
+    def __str__(self):
+        """Human representation of a course run translation."""
+        return "{model}: {name}".format(
+            model=self._meta.verbose_name.title(), name=self.title
         )
 
+    # pylint: disable=signature-differs
+    def save(self, *args, **kwargs):
+        """
+        Mark related course page dirty if the title has changed compared to the public version.
+        """
+        if (
+            # Does the public translation have a different title?
+            self.__class__.objects.filter(
+                master__draft_course_run__translations__pk=self.pk,
+                language_code=self.language_code,
+            )
+            .exclude(title=self.title)
+            .exists()
+        ):
+            self.master.direct_course.extended_object.title_set.filter(
+                language=self.language_code
+            ).update(
+                publisher_state=PUBLISHER_STATE_DIRTY
+            )  # mark page dirty
+
+        return super().save(*args, **kwargs)
+
 
 class CoursePluginModel(PagePluginMixin, CMSPlugin):
     """
@@ -585,6 +869,7 @@ class CoursePluginModel(PagePluginMixin, CMSPlugin):
     class Meta:
         db_table = "richie_course_plugin"
         verbose_name = _("course plugin")
+        verbose_name_plural = _("course plugins")
 
 
 class Licence(TranslatableModel):
@@ -610,6 +895,7 @@ class Licence(TranslatableModel):
     class Meta:
         db_table = "richie_licence"
         verbose_name = _("licence")
+        verbose_name_plural = _("licences")
 
     def __str__(self):
         """Human representation of a licence."""
@@ -633,6 +919,7 @@ class LicenceTranslation(TranslatedFieldsModel):
         db_table = "richie_licence_translation"
         unique_together = ("language_code", "master")
         verbose_name = _("Licence translation")
+        verbose_name_plural = _("licence translations")
 
     def __str__(self):
         """Human representation of a licence translation."""
@@ -652,6 +939,7 @@ class LicencePluginModel(CMSPlugin):
     class Meta:
         db_table = "richie_licence_plugin"
         verbose_name = _("licence plugin")
+        verbose_name_plural = _("licence plugins")
 
     def __str__(self):
         """Human representation of a licence plugin."""
@@ -661,4 +949,3 @@ class LicencePluginModel(CMSPlugin):
 
 
 extension_pool.register(Course)
-extension_pool.register(CourseRun)
