@@ -16,7 +16,10 @@ Otherwise, you can just use the usual Django pattern in your settings.py file:
     from richie.apps.courses.settings import *
     ```
 """
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
+
+# Associated LMS backends
+RICHIE_LMS_BACKENDS = []
 
 # Easy Thumbnails
 THUMBNAIL_PROCESSORS = (
@@ -30,7 +33,6 @@ THUMBNAIL_PROCESSORS = (
 # Django CMS
 CMS_TEMPLATES = (
     ("courses/cms/course_detail.html", _("Course page")),
-    ("courses/cms/course_run_detail.html", _("Course run page")),
     ("courses/cms/organization_list.html", _("Organization list")),
     ("courses/cms/organization_detail.html", _("Organization page")),
     ("courses/cms/category_list.html", _("Category list")),
@@ -52,8 +54,13 @@ CMS_PLACEHOLDER_CONF = {
     # Footer
     "footer": {
         "name": _("Footer"),
-        "plugins": ["LinkPlugin", "SectionPlugin"],
-        "child_classes": {"SectionPlugin": ["LinkPlugin", "SectionPlugin"]},
+        "plugins": ["NestedItemPlugin", "LinkPlugin"],
+        "NestedItemPlugin": ["LinkPlugin"],
+    },
+    "static_blogpost_headline": {
+        "name": _("Static headline"),
+        "plugins": ["SectionPlugin", "CKEditorPlugin"],
+        "child_classes": {"SectionPlugin": ["CKEditorPlugin"]},
     },
     # -- Page Placeholders
     # Homepage
@@ -63,37 +70,49 @@ CMS_PLACEHOLDER_CONF = {
         "child_classes": {
             "SectionPlugin": [
                 "BlogPostPlugin",
-                "CoursePlugin",
                 "CategoryPlugin",
+                "CoursePlugin",
+                "CKEditorPlugin",
+                "GlimpsePlugin",
                 "LinkPlugin",
-                "OrganizationPlugin",
+                "NestedItemPlugin",
                 "OrganizationsByCategoryPlugin",
+                "OrganizationPlugin",
                 "PersonPlugin",
-            ]
+                "ProgramPlugin",
+                "SectionPlugin",
+            ],
+            "NestedItemPlugin": ["CategoryPlugin"],
         },
     },
     # Single column page
-    "richie/single-column.html maincontent": {
+    "richie/single_column.html maincontent": {
         "name": _("Main content"),
         "excluded_plugins": ["CKEditorPlugin", "GoogleMapPlugin"],
         "parent_classes": {
             "BlogPostPlugin": ["SectionPlugin"],
             "CategoryPlugin": ["SectionPlugin"],
             "CoursePlugin": ["SectionPlugin"],
+            "GlimpsePlugin": ["SectionPlugin"],
             "OrganizationPlugin": ["SectionPlugin"],
             "OrganizationsByCategoryPlugin": ["SectionPlugin"],
             "PersonPlugin": ["SectionPlugin"],
+            "ProgramPlugin": ["SectionPlugin"],
         },
         "child_classes": {
             "SectionPlugin": [
                 "BlogPostPlugin",
                 "CategoryPlugin",
                 "CoursePlugin",
+                "GlimpsePlugin",
                 "LinkPlugin",
-                "OrganizationPlugin",
+                "NestedItemPlugin",
                 "OrganizationsByCategoryPlugin",
+                "OrganizationPlugin",
                 "PersonPlugin",
-            ]
+                "ProgramPlugin",
+            ],
+            "NestedItemPlugin": ["NestedItemPlugin", "LinkPlugin"],
         },
     },
     # Course detail
@@ -115,6 +134,7 @@ CMS_PLACEHOLDER_CONF = {
     "courses/cms/course_detail.html course_description": {
         "name": _("About the course"),
         "plugins": ["CKEditorPlugin"],
+        "limits": {"CKEditorPlugin": 1},
     },
     "courses/cms/course_detail.html course_skills": {
         "name": _("What you will learn"),
@@ -134,7 +154,8 @@ CMS_PLACEHOLDER_CONF = {
     },
     "courses/cms/course_detail.html course_plan": {
         "name": _("Plan"),
-        "plugins": ["CKEditorPlugin"],
+        "plugins": ["NestedItemPlugin"],
+        "child_classes": {"NestedItemPlugin": ["NestedItemPlugin"]},
     },
     "courses/cms/course_detail.html course_information": {
         "name": _("Complementary information"),
@@ -142,8 +163,11 @@ CMS_PLACEHOLDER_CONF = {
         "parent_classes": {
             "CKEditorPlugin": ["SectionPlugin"],
             "SimplePicturePlugin": ["SectionPlugin"],
+            "GlimpsePlugin": ["SectionPlugin"],
+        },
+        "child_classes": {
+            "SectionPlugin": ["CKEditorPlugin", "SimplePicturePlugin", "GlimpsePlugin"]
         },
-        "child_classes": {"SectionPlugin": ["CKEditorPlugin", "SimplePicturePlugin"]},
     },
     "courses/cms/course_detail.html course_license_content": {
         "name": _("License for the course content"),
@@ -228,6 +252,14 @@ CMS_PLACEHOLDER_CONF = {
         "plugins": ["PlainTextPlugin"],
         "limits": {"PlainTextPlugin": 1},
     },
+    "courses/cms/person_detail.html maincontent": {
+        "name": _("Main Content"),
+        "plugins": ["CKEditorPlugin", "PersonPlugin", "SectionPlugin", "GlimpsePlugin"],
+        "child_classes": {
+            "SectionPlugin": ["CKEditorPlugin", "GlimpsePlugin", "PersonPlugin"]
+        },
+        "limits": {"CKEditorPlugin": 1},
+    },
     "courses/cms/person_detail.html organizations": {
         "name": _("Organizations"),
         "plugins": ["OrganizationPlugin"],
@@ -256,6 +288,11 @@ CMS_PLACEHOLDER_CONF = {
         "name": _("Body"),
         "excluded_plugins": ["CKEditorPlugin", "GoogleMapPlugin"],
     },
+    "courses/cms/blogpost_detail.html headline": {
+        "name": _("Headline"),
+        "plugins": ["SectionPlugin", "CKEditorPlugin"],
+        "child_classes": {"SectionPlugin": ["CKEditorPlugin"]},
+    },
     # Program page detail
     "courses/cms/program_detail.html program_cover": {
         "name": _("Cover"),
@@ -269,12 +306,33 @@ CMS_PLACEHOLDER_CONF = {
     },
     "courses/cms/program_detail.html program_body": {
         "name": _("Body"),
-        "excluded_plugins": ["CKEditorPlugin", "GoogleMapPlugin"],
+        "plugins": ["CKEditorPlugin"],
+        "limits": {"CKEditorPlugin": 1},
     },
     "courses/cms/program_detail.html program_courses": {
         "name": _("Courses"),
         "plugins": ["CoursePlugin"],
     },
+    "courses/cms/program_list.html maincontent": {
+        "name": _("Main content"),
+        "plugins": ["SectionPlugin"],
+        "child_classes": {
+            "SectionPlugin": [
+                "BlogPostPlugin",
+                "CategoryPlugin",
+                "CoursePlugin",
+                "GlimpsePlugin",
+                "LinkPlugin",
+                "OrganizationPlugin",
+                "OrganizationsByCategoryPlugin",
+                "PersonPlugin",
+                "CKEditorPlugin",
+                "SectionPlugin",
+                "NestedItemPlugin",
+            ],
+            "NestedItemPlugin": ["CategoryPlugin"],
+        },
+    },
 }
 
 # Main CKEditor configuration
@@ -351,6 +409,30 @@ CKEDITOR_LIMITED_CONFIGURATION = {
     ],
 }
 
+# CKEditor configuration for formatting section title:
+# only bold entity
+CKEDITOR_INLINE_BOLD_CONFIGURATION = {
+    "language": "{{ language }}",
+    "skin": "moono-lisa",
+    "toolbarCanCollapse": False,
+    "contentsCss": "/static/css/ckeditor.css",
+    # Only enable following tag definitions
+    "allowedContent": ["strong"],
+    # Block commands which adds break lines (Enter & Shift + Enter)
+    # Enter Key Code = 13
+    # CKEDITOR.SHIFT + Enter = 2228224 + 13 = 2228237
+    "blockedKeystrokes": [13, 2228237],
+    "keystrokes": [[13, None], [2228237, None]],
+    # Enabled showblocks as default behavior
+    "startupOutlineBlocks": True,
+    # Default toolbar configurations for djangocms_text_ckeditor
+    "toolbar_HTMLField": [["Undo", "Redo"], ["Bold"]],
+    "enterMode": 2,
+    "autoParagraph": False,
+    "resize_enabled": False,
+    "height": 68,
+}
+
 # Additional LinkPlugin templates. Note how choice value is just a keyword
 # instead of full template path. Value is used inside a path formatting
 # such as "templates/djangocms_link/VALUE/link.html"
@@ -372,6 +454,11 @@ RICHIE_SIMPLETEXT_CONFIGURATION = [
         "ckeditor": "CKEDITOR_LIMITED_CONFIGURATION",
         "max_length": 1200,
     },
+    {
+        "placeholders": ["maincontent"],
+        "ckeditor": "CKEDITOR_SETTINGS",
+        "max_length": 5000,
+    },
     {
         "placeholders": ["course_assessment", "course_format", "course_prerequisites"],
         "ckeditor": "CKEDITOR_BASIC_CONFIGURATION",
