@@ -7,6 +7,8 @@ from django.conf import settings
 from django.contrib.sites.models import Site
 from django.core.management.base import BaseCommand, CommandError
 from django.test.utils import override_settings
+from django.utils import translation
+from django.utils.translation import gettext_lazy as _
 
 import factory
 from cms.api import add_plugin
@@ -14,30 +16,11 @@ from cms.models import StaticPlaceholder
 
 from richie.apps.core.factories import create_text_plugin, image_getter
 from richie.apps.core.helpers import recursive_page_creation
-from richie.apps.courses.factories import (
-    VIDEO_SAMPLE_LINKS,
-    BlogPostFactory,
-    CourseFactory,
-    CourseRunFactory,
-    LicenceFactory,
-    OrganizationFactory,
-    PersonFactory,
-    ProgramFactory,
-)
+from richie.apps.courses import factories, models
+from richie.plugins.glimpse import defaults as glimpse_defaults
+from richie.plugins.glimpse.factories import GlimpseFactory
 
-from ...defaults import (
-    FOOTER_CONTENT,
-    HOMEPAGE_CONTENT,
-    ICONS_INFO,
-    LEVELS_INFO,
-    NB_OBJECTS,
-    PAGES_INFO,
-    PARTNERSHIPS_INFO,
-    SINGLECOLUMN_CONTENT,
-    SITEMAP_PAGE_PARAMS,
-    SUBJECTS_INFO,
-    TAGS_INFO,
-)
+from ... import defaults
 from ...helpers import create_categories
 from ...utils import pick_image
 
@@ -49,7 +32,7 @@ def get_number_of_course_runs():
     Returns a random integer between 1 and the max number of course runs.
     We make it a convenience method so that it can be mocked in tests.
     """
-    return random.randint(1, NB_OBJECTS["course_courseruns"])  # nosec
+    return random.randint(1, defaults.NB_OBJECTS["course_courseruns"])  # nosec
 
 
 def get_number_of_icons():
@@ -60,7 +43,8 @@ def get_number_of_icons():
     We make it a convenience method so that it can be mocked in tests.
     """
     return random.choice(  # nosec
-        [0] * NB_OBJECTS["course_icons"] + list(range(NB_OBJECTS["course_icons"]))
+        [0] * defaults.NB_OBJECTS["course_icons"]
+        + list(range(defaults.NB_OBJECTS["course_icons"]))
     )
 
 
@@ -71,6 +55,8 @@ def get_number_of_icons():
 # We choose to ignore this false positive warning.
 
 # pylint: disable=too-many-locals,too-many-statements,too-many-branches
+
+
 @override_settings(RICHIE_KEEP_SEARCH_UPDATED=False)
 def create_demo_site():
     """
@@ -80,99 +66,118 @@ def create_demo_site():
     plugins to each page.
     """
     site = Site.objects.get(id=1)
+    site.domain = getattr(
+        settings, "RICHIE_DEMO_SITE_DOMAIN", defaults.DEFAULT_DEMO_SITE_DOMAIN
+    )
+    site.name = "Richie demonstration"
+    site.save()
+
+    lms_endpoint = (
+        getattr(settings, "RICHIE_LMS_BACKENDS", None)
+        or [{"BASE_URL": defaults.DEFAULT_LMS_ENDPOINT}]
+    )[0]["BASE_URL"]
 
     # Create pages as described in PAGES_INFOS
-    pages_created = recursive_page_creation(site, PAGES_INFO)
+    pages_created = recursive_page_creation(site, defaults.PAGES_INFO)
 
     # Create the footer links
+    def create_footer_link(**link_info):
+        """
+        Use LinkPlugin to create a link in footer menu with link_info
+
+        Links can be nested into a NestedItemPlugin, in this case link_info contains
+        a target key.
+        """
+        if "internal_link" in link_info:
+            link_info = link_info.copy()
+            link_info["internal_link"] = pages_created[link_info["internal_link"]]
+        add_plugin(plugin_type="LinkPlugin", **link_info)
+
     footer_static_ph = StaticPlaceholder.objects.get_or_create(code="footer")[0]
     for footer_placeholder in [footer_static_ph.draft, footer_static_ph.public]:
-        for language, content in FOOTER_CONTENT.items():
-            # Create the <ul> section to carry the list of links
-            section_plugin = add_plugin(
-                footer_placeholder,
-                plugin_type="SectionPlugin",
-                language=language,
-                template="richie/section/section_list.html",
-            )
-
-            # One column per content object
+        for language, content in defaults.FOOTER_CONTENT.items():
             for footer_info in content:
-                column_plugin = add_plugin(
-                    footer_placeholder,
-                    plugin_type="SectionPlugin",
-                    language=language,
-                    target=section_plugin,
-                    template="richie/section/section_list.html",
-                    title=footer_info.get("title"),
-                )
-                for item_info in footer_info.get("items", []):
-                    if "internal_link" in item_info:
-                        item_info = item_info.copy()
-                        item_info["internal_link"] = pages_created[
-                            item_info["internal_link"]
-                        ]
-                    add_plugin(
+                if "items" in footer_info:
+                    # Create the first level items for main columns
+                    nest_column_plugin = add_plugin(
                         footer_placeholder,
-                        plugin_type="LinkPlugin",
+                        plugin_type="NestedItemPlugin",
                         language=language,
-                        target=column_plugin,
-                        **item_info,
+                        content=footer_info.get("title", ""),
+                    )
+
+                    # Create the second level items for links
+                    for item_info in footer_info.get("items", []):
+                        create_footer_link(
+                            language=language,
+                            placeholder=footer_placeholder,
+                            target=nest_column_plugin,
+                            **item_info,
+                        )
+                else:
+                    # Create link at first level
+                    create_footer_link(
+                        language=language, placeholder=footer_placeholder, **footer_info
                     )
 
     # Create some licences
-    licences = LicenceFactory.create_batch(
-        NB_OBJECTS["licences"], logo__file__from_path=pick_image("licence")()
+    licences = (
+        factories.LicenceFactory.create_batch(
+            defaults.NB_OBJECTS["licences"],
+            logo__file__from_path=pick_image("licence")(),
+        )
+        if defaults.NB_OBJECTS.get("licences")
+        else []
     )
 
     # Generate each category tree and return a list of the leaf categories
     icons = list(
         create_categories(
-            **ICONS_INFO,
+            **defaults.ICONS_INFO,
             fill_banner=pick_image("banner"),
-            fill_logo=pick_image("logo"),
+            fill_logo=pick_image("category_logo"),
             page_parent=pages_created["categories"],
         )
     )
     levels = list(
         create_categories(
-            **LEVELS_INFO,
+            **defaults.LEVELS_INFO,
             fill_banner=pick_image("banner"),
-            fill_logo=pick_image("logo"),
+            fill_logo=pick_image("category_logo"),
             page_parent=pages_created["categories"],
         )
     )
     subjects = list(
         create_categories(
-            **SUBJECTS_INFO,
+            **defaults.SUBJECTS_INFO,
             fill_banner=pick_image("banner"),
-            fill_logo=pick_image("logo"),
+            fill_logo=pick_image("category_logo"),
             page_parent=pages_created["categories"],
         )
     )
     partnerships = list(
         create_categories(
-            **PARTNERSHIPS_INFO,
+            **defaults.PARTNERSHIPS_INFO,
             fill_banner=pick_image("banner"),
-            fill_logo=pick_image("logo"),
+            fill_logo=pick_image("category_logo"),
             page_parent=pages_created["categories"],
         )
     )
     tags = list(
         create_categories(
-            **TAGS_INFO,
+            **defaults.TAGS_INFO,
             fill_banner=pick_image("banner"),
-            fill_logo=pick_image("logo"),
+            fill_logo=pick_image("category_logo"),
             page_parent=pages_created["categories"],
         )
     )
 
     # Create organizations under the `Organizations` page
     organizations = []
-    for i in range(NB_OBJECTS["organizations"]):
+    for i in range(defaults.NB_OBJECTS["organizations"]):
         # Randomly assign each organization to a partnership level category
         organizations.append(
-            OrganizationFactory(
+            factories.OrganizationFactory(
                 page_in_navigation=True,
                 page_languages=["en", "fr"],
                 page_parent=pages_created["organizations"],
@@ -190,22 +195,24 @@ def create_demo_site():
     # Create persons under the `persons` page
     persons = []
     persons_for_organization = defaultdict(list)
-    for _ in range(NB_OBJECTS["persons"]):
+    for _i in range(defaults.NB_OBJECTS["persons"]):
         # Randomly assign each person to a set of organizations
         person_organizations = random.sample(
             organizations,
-            random.randint(1, NB_OBJECTS["person_organizations"]),  # nosec
+            random.randint(1, defaults.NB_OBJECTS["person_organizations"]),  # nosec
         )
-        person = PersonFactory(
+        person = factories.PersonFactory(
             page_in_navigation=True,
             page_languages=["en", "fr"],
             page_parent=pages_created["persons"],
             fill_categories=random.sample(
-                subjects, random.randint(1, NB_OBJECTS["person_subjects"])  # nosec
+                subjects,
+                random.randint(1, defaults.NB_OBJECTS["person_subjects"]),  # nosec
             ),
             fill_organizations=person_organizations,
             fill_portrait=pick_image("portrait"),
             fill_bio=True,
+            fill_maincontent=True,
             should_publish=True,
         )
         persons.append(person)
@@ -219,12 +226,12 @@ def create_demo_site():
     # Create courses under the `Course` page with categories and organizations
     # relations
     courses = []
-    for _ in range(NB_OBJECTS["courses"]):
-        video_sample = random.choice(VIDEO_SAMPLE_LINKS)  # nosec
+    for _i in range(defaults.NB_OBJECTS["courses"]):
+        video_sample = random.choice(factories.VIDEO_SAMPLE_LINKS)  # nosec
 
         # Randomly assign each course to a set of organizations
         course_organizations = random.sample(
-            organizations, NB_OBJECTS["course_organizations"]
+            organizations, defaults.NB_OBJECTS["course_organizations"]
         )
 
         # Only the persons members of these organizations are eligible to be part
@@ -235,18 +242,24 @@ def create_demo_site():
             for person in persons_for_organization[o.id]
         )
 
-        course = CourseFactory(
+        course_licences = (
+            [
+                ("course_license_content", random.choice(licences)),  # nosec
+                ("course_license_participation", random.choice(licences)),  # nosec
+            ]
+            if licences
+            else []
+        )
+
+        course = factories.CourseFactory(
             page_in_navigation=True,
             page_languages=["en", "fr"],
             page_parent=pages_created["courses"],
-            fill_licences=[
-                ("course_license_content", random.choice(licences)),  # nosec
-                ("course_license_participation", random.choice(licences)),  # nosec
-            ],
+            fill_licences=course_licences,
             fill_team=random.sample(
                 eligible_persons,
                 min(
-                    random.randint(1, NB_OBJECTS["course_persons"]),  # nosec
+                    random.randint(1, defaults.NB_OBJECTS["course_persons"]),  # nosec
                     len(eligible_persons),
                 ),
             ),
@@ -254,25 +267,97 @@ def create_demo_site():
             fill_cover=pick_image("cover")(video_sample.image),
             fill_categories=[
                 *random.sample(
-                    subjects, random.randint(1, NB_OBJECTS["course_subjects"])  # nosec
+                    subjects,
+                    random.randint(1, defaults.NB_OBJECTS["course_subjects"]),  # nosec
                 ),
                 random.choice(levels),  # nosec
             ],
             fill_icons=random.sample(icons, get_number_of_icons()),
             fill_organizations=course_organizations,
-            fill_texts=[
-                "course_description",
-                "course_format",
-                "course_prerequisites",
-                "course_plan",
-                # "course_license_content",
-                # "course_license_participation",
-            ],
-            should_publish=True,
+            fill_plan=True,
+            fill_texts={
+                "course_assessment": "CKEditorPlugin",
+                "course_description": "CKEditorPlugin",
+                "course_introduction": "PlainTextPlugin",
+                "course_format": "CKEditorPlugin",
+                "course_prerequisites": "CKEditorPlugin",
+                "course_skills": "CKEditorPlugin",
+            },
         )
         course.create_permissions_for_organization(course_organizations[0])
         courses.append(course)
 
+        # Add extra information
+        for language in course.extended_object.get_languages():
+            placeholder = course.extended_object.placeholders.get(
+                slot="course_information"
+            )
+            nb_half_rows = 2 * random.randint(0, 1)  # nosec
+            nb_full_rows = random.randint(0, 2)  # nosec
+            nb_cards = 4 * random.randint(0, 1)  # nosec
+
+            # Partners
+            if nb_half_rows or nb_full_rows:
+                partner_section = add_plugin(
+                    language=language,
+                    placeholder=placeholder,
+                    plugin_type="SectionPlugin",
+                    title=defaults.COURSE_CONTENT[language]["partners_title"],
+                )
+            for _i in range(nb_half_rows):
+                glimpse_data = factory.build(
+                    dict,
+                    FACTORY_CLASS=GlimpseFactory,
+                    variant=glimpse_defaults.ROW_HALF,
+                    image=image_getter(pick_image("logo")()),
+                )
+                glimpse_data["image"].save()
+                add_plugin(
+                    language=language,
+                    placeholder=placeholder,
+                    plugin_type="GlimpsePlugin",
+                    target=partner_section,
+                    **glimpse_data,
+                )
+            for _i in range(nb_full_rows):
+                glimpse_data = factory.build(
+                    dict,
+                    FACTORY_CLASS=GlimpseFactory,
+                    variant=glimpse_defaults.ROW_FULL,
+                    image=image_getter(pick_image("logo")()),
+                )
+                glimpse_data["image"].save()
+                add_plugin(
+                    language=language,
+                    placeholder=placeholder,
+                    plugin_type="GlimpsePlugin",
+                    target=partner_section,
+                    **glimpse_data,
+                )
+            # Sponsors
+            if nb_cards:
+                sponsor_section = add_plugin(
+                    language=language,
+                    placeholder=placeholder,
+                    plugin_type="SectionPlugin",
+                    title=defaults.COURSE_CONTENT[language]["sponsors_title"],
+                )
+            for _i in range(nb_cards):
+                glimpse_data = factory.build(
+                    dict,
+                    FACTORY_CLASS=GlimpseFactory,
+                    variant=glimpse_defaults.CARD_SQUARE,
+                    image=image_getter(pick_image("logo")()),
+                )
+                glimpse_data["image"].save()
+                add_plugin(
+                    language=language,
+                    placeholder=placeholder,
+                    plugin_type="GlimpsePlugin",
+                    target=sponsor_section,
+                    **glimpse_data,
+                )
+
         # Add a random number of course runs to the course
         nb_course_runs = get_number_of_course_runs()
         # pick a subset of languages for this course (otherwise all courses will have more or
@@ -281,21 +366,30 @@ def create_demo_site():
             ["de", "en", "es", "fr", "it", "nl"], random.randint(1, 4)  # nosec
         )
         for i in range(nb_course_runs):
-            CourseRunFactory(
+            course_run = factories.CourseRunFactory(
                 __sequence=i,
                 languages=random.sample(
                     languages_subset, random.randint(1, len(languages_subset))  # nosec
                 ),
-                page_in_navigation=False,
-                page_languages=["en", "fr"],
-                page_parent=course.extended_object,
-                should_publish=True,
+                direct_course=course,
+                resource_link=f"{lms_endpoint}/courses/course-v1:edX+DemoX+Demo_Course/info",
             )
+            for language in course.extended_object.get_languages():
+                with translation.override(language):
+                    models.CourseRunTranslation.objects.update_or_create(
+                        master=course_run,
+                        language_code=language,
+                        defaults={"title": _(f"Run {i:d}")},
+                    )
+
+        # Publish the course in all languages
+        for language in course.extended_object.get_languages():
+            course.extended_object.publish(language)
 
     # Create blog posts under the `News` page
     blogposts = []
-    for _ in range(NB_OBJECTS["blogposts"]):
-        post = BlogPostFactory.create(
+    for _i in range(defaults.NB_OBJECTS["blogposts"]):
+        post = factories.BlogPostFactory.create(
             page_in_navigation=True,
             page_languages=["en", "fr"],
             page_parent=pages_created["blogposts"],
@@ -303,8 +397,8 @@ def create_demo_site():
             fill_excerpt=True,
             fill_body=True,
             fill_categories=[
-                *random.sample(levels, NB_OBJECTS["blogpost_levels"]),
-                *random.sample(tags, NB_OBJECTS["blogpost_tags"]),
+                *random.sample(levels, defaults.NB_OBJECTS["blogpost_levels"]),
+                *random.sample(tags, defaults.NB_OBJECTS["blogpost_tags"]),
             ],
             fill_author=random.sample(persons, 1),
             should_publish=True,
@@ -313,19 +407,36 @@ def create_demo_site():
 
     # Create programs under the `Programs` page
     programs = []
-    for _ in range(NB_OBJECTS["programs"]):
-        program = ProgramFactory.create(
+    for _i in range(defaults.NB_OBJECTS["programs"]):
+        program = factories.ProgramFactory.create(
             page_in_navigation=True,
             page_languages=["en", "fr"],
             page_parent=pages_created["programs"],
             fill_cover=pick_image("cover"),
             fill_excerpt=True,
             fill_body=True,
-            fill_courses=[*random.sample(courses, NB_OBJECTS["programs_courses"])],
+            fill_courses=[
+                *random.sample(courses, defaults.NB_OBJECTS["programs_courses"])
+            ],
             should_publish=True,
         )
         programs.append(program)
 
+    # Create some content on the programs list page
+    placeholder = pages_created["programs"].placeholders.get(slot="maincontent")
+
+    for language in pages_created["programs"].get_languages():
+        create_text_plugin(
+            pages_created["programs"],
+            placeholder,
+            nb_paragraphs=random.randint(3, 4),  # nosec
+            languages=[language],
+            plugin_type="TextPlugin",
+        )
+
+        # Once content has been added we must publish again homepage
+        pages_created["programs"].publish(language)
+
     # Once everything has been created, use some content to create a homepage
     placeholder = pages_created["home"].placeholders.get(slot="maincontent")
 
@@ -336,7 +447,7 @@ def create_demo_site():
     logo = image_getter(pick_image("logo")())
 
     # - Create the home page in each language
-    for language, content in HOMEPAGE_CONTENT.items():
+    for language, content in defaults.HOMEPAGE_CONTENT.items():
         # Add a banner
         add_plugin(
             language=language,
@@ -344,8 +455,6 @@ def create_demo_site():
             plugin_type="LargeBannerPlugin",
             title=content["banner_title"],
             background_image=banner,
-            logo=logo,
-            logo_alt_text="logo",
             content=content["banner_content"],
             template=content["banner_template"],
         )
@@ -357,7 +466,7 @@ def create_demo_site():
             title=content["courses_title"],
             template=content["section_template"],
         )
-        for course in random.sample(courses, NB_OBJECTS["home_courses"]):
+        for course in random.sample(courses, defaults.NB_OBJECTS["home_courses"]):
             add_plugin(
                 language=language,
                 placeholder=placeholder,
@@ -383,7 +492,7 @@ def create_demo_site():
             title=content["blogposts_title"],
             template=content["section_template"],
         )
-        for blogpost in random.sample(blogposts, NB_OBJECTS["home_blogposts"]):
+        for blogpost in random.sample(blogposts, defaults.NB_OBJECTS["home_blogposts"]):
             add_plugin(
                 language=language,
                 placeholder=placeholder,
@@ -409,7 +518,7 @@ def create_demo_site():
             title=content["programs_title"],
             template=content["section_template"],
         )
-        for program in random.sample(programs, NB_OBJECTS["home_programs"]):
+        for program in random.sample(programs, defaults.NB_OBJECTS["home_programs"]):
             add_plugin(
                 language=language,
                 placeholder=placeholder,
@@ -436,7 +545,7 @@ def create_demo_site():
             template=content["section_template"],
         )
         for organization in random.sample(
-            organizations, NB_OBJECTS["home_organizations"]
+            organizations, defaults.NB_OBJECTS["home_organizations"]
         ):
             add_plugin(
                 language=language,
@@ -463,7 +572,7 @@ def create_demo_site():
             title=content["subjects_title"],
             template=content["section_template"],
         )
-        for subject in random.sample(subjects, NB_OBJECTS["home_subjects"]):
+        for subject in random.sample(subjects, defaults.NB_OBJECTS["home_subjects"]):
             add_plugin(
                 language=language,
                 placeholder=placeholder,
@@ -489,7 +598,7 @@ def create_demo_site():
             title=content["persons_title"],
             template=content["section_template"],
         )
-        for person in random.sample(persons, NB_OBJECTS["home_persons"]):
+        for person in random.sample(persons, defaults.NB_OBJECTS["home_persons"]):
             add_plugin(
                 language=language,
                 placeholder=placeholder,
@@ -507,6 +616,54 @@ def create_demo_site():
             internal_link=pages_created["persons"],
         )
 
+        # Add Glimpse quotes with empty title
+        quotes_section = add_plugin(
+            language=language,
+            placeholder=placeholder,
+            plugin_type="SectionPlugin",
+            title="",
+        )
+        for _i in range(3):
+            glimpse_data = factory.build(
+                dict,
+                FACTORY_CLASS=GlimpseFactory,
+                variant=glimpse_defaults.QUOTE,
+                title=None,
+                image=image_getter(pick_image("portrait")()),
+            )
+            glimpse_data["image"].save()
+            add_plugin(
+                language=language,
+                placeholder=placeholder,
+                plugin_type="GlimpsePlugin",
+                target=quotes_section,
+                **glimpse_data,
+            )
+
+        # Add Glimpse cards with empty content
+        cards_section = add_plugin(
+            language=language,
+            placeholder=placeholder,
+            plugin_type="SectionPlugin",
+            title="",
+        )
+        for _i in range(4):
+            glimpse_data = factory.build(
+                dict,
+                FACTORY_CLASS=GlimpseFactory,
+                variant=glimpse_defaults.CARD_SQUARE,
+                content=None,
+                image=image_getter(pick_image("cover")()),
+            )
+            glimpse_data["image"].save()
+            add_plugin(
+                language=language,
+                placeholder=placeholder,
+                plugin_type="GlimpsePlugin",
+                target=cards_section,
+                **glimpse_data,
+            )
+
         # Once content has been added we must publish again homepage
         pages_created["home"].publish(language)
 
@@ -520,10 +677,10 @@ def create_demo_site():
     logo = image_getter(pick_image("logo")())
 
     # - Get a video
-    video_sample = random.choice(VIDEO_SAMPLE_LINKS)  # nosec
+    video_sample = random.choice(factories.VIDEO_SAMPLE_LINKS)  # nosec
 
     # - Create sample page in each language
-    for language, content in SINGLECOLUMN_CONTENT.items():
+    for language, content in defaults.SINGLECOLUMN_CONTENT.items():
         # Add a banner
         add_plugin(
             language=language,
@@ -597,12 +754,13 @@ def create_demo_site():
             internal_link=pages_created["home"],
         )
         # Add a licence
-        add_plugin(
-            language=language,
-            placeholder=placeholder,
-            plugin_type="LicencePlugin",
-            licence=random.choice(licences),  # nosec
-        )
+        if licences:
+            add_plugin(
+                language=language,
+                placeholder=placeholder,
+                plugin_type="LicencePlugin",
+                licence=random.choice(licences),  # nosec
+            )
         # Add a simple picture entry
         add_plugin(
             language=language,
@@ -612,8 +770,8 @@ def create_demo_site():
         )
         # Add a plain text
         text = factory.Faker(
-            "text", max_nb_chars=random.randint(150, 250), locale=language  # nosec
-        ).generate({})
+            "text", max_nb_chars=random.randint(150, 250)  # nosec
+        ).evaluate(None, None, {"locale": language})
         add_plugin(
             language=language,
             placeholder=placeholder,
@@ -631,7 +789,7 @@ def create_demo_site():
         parent_instance = add_plugin(
             language=language, placeholder=placeholder, plugin_type="HTMLSitemapPlugin"
         )
-        for name, params in SITEMAP_PAGE_PARAMS.items():
+        for name, params in defaults.SITEMAP_PAGE_PARAMS.items():
             add_plugin(
                 language=language,
                 placeholder=placeholder,
