@@ -21,7 +21,6 @@ from ..forms import CourseSearchForm
 from ..text_indexing import MULTILINGUAL_TEXT
 from ..utils.i18n import get_best_field_language
 from ..utils.indexers import slice_string_for_completion
-from . import ES_INDICES
 
 
 class CoursesIndexer:
@@ -67,6 +66,7 @@ class CoursesIndexer:
                 for lang, _ in settings.LANGUAGES
             },
             "is_new": {"type": "boolean"},
+            "is_listed": {"type": "boolean"},
             # Not searchable
             "absolute_url": {"type": "object", "enabled": False},
             "cover_image": {"type": "object", "enabled": False},
@@ -77,6 +77,8 @@ class CoursesIndexer:
         "absolute_url",
         "categories",
         "cover_image",
+        "duration",
+        "effort",
         "icon",
         "organization_highlighted",
         "organizations",
@@ -411,6 +413,18 @@ class CoursesIndexer:
         ):
             descriptions[simple_text.cmsplugin_ptr.language].append(simple_text.body)
 
+        # Prepare localized duration texts
+        duration = {}
+        for language, _ in settings.LANGUAGES:
+            with translation.override(language):
+                duration[language] = course.get_duration_display()
+
+        # Prepare localized effort texts
+        effort = {}
+        for language, _ in settings.LANGUAGES:
+            with translation.override(language):
+                effort[language] = course.get_effort_display()
+
         # Prepare categories, making sure we get title information for categories
         # in the same query
         category_pages = (
@@ -488,9 +502,7 @@ class CoursesIndexer:
                 language: course.extended_object.get_absolute_url(language)
                 for language in titles.keys()
             },
-            "categories": [
-                ES_INDICES.categories.get_es_id(page) for page in category_pages
-            ],
+            "categories": [page.category.get_es_id() for page in category_pages],
             # Index the names of categories to surface them in full text searches
             "categories_names": reduce(
                 lambda acc, title: {
@@ -505,12 +517,19 @@ class CoursesIndexer:
             "complete": {
                 language: slice_string_for_completion(title)
                 for language, title in titles.items()
-            },
+            }
+            if course.is_listed
+            else None,
             "course_runs": course_runs,
             "cover_image": cover_images,
-            "description": {l: " ".join(st) for l, st in descriptions.items()},
+            "description": {
+                language: " ".join(st) for language, st in descriptions.items()
+            },
+            "duration": duration,
+            "effort": effort,
             "icon": icon_images,
             "is_new": len(course_runs) == 1,
+            "is_listed": course.is_listed,
             # Pick the highlighted organization from the organizations QuerySet to benefit from
             # the prefetch of related title sets
             "organization_highlighted": {
@@ -520,8 +539,7 @@ class CoursesIndexer:
             if organization_highlighted
             else None,
             "organizations": [
-                ES_INDICES.organizations.get_es_id(o.extended_object)
-                for o in organizations
+                organization.get_es_id() for organization in organizations
             ],
             # Index the names of organizations to surface them in full text searches
             "organizations_names": reduce(
@@ -565,11 +583,15 @@ class CoursesIndexer:
         """
         index = index or cls.index_name
 
-        for course in Course.objects.filter(
-            extended_object__publisher_is_draft=False,  # index the public object
-            extended_object__title_set__published=True,  # only index published courses
-            extended_object__node__parent__cms_pages__course__isnull=True,  # exclude snapshots
-        ).distinct():
+        for course in (
+            Course.objects.filter(
+                extended_object__publisher_is_draft=False,  # index the public object
+                extended_object__title_set__published=True,  # only index published courses
+                extended_object__node__parent__cms_pages__course__isnull=True,  # exclude snapshots
+            )
+            .distinct()
+            .iterator()
+        ):
             yield cls.get_es_document_for_course(course, index=index, action=action)
 
     @staticmethod
@@ -589,11 +611,19 @@ class CoursesIndexer:
             state["date_time"] = None
 
         return {
+            **{
+                field: get_best_field_language(source[field], language)
+                for field in [
+                    "absolute_url",
+                    "cover_image",
+                    "duration",
+                    "effort",
+                    "icon",
+                    "title",
+                ]
+            },
             "id": es_course["_id"],
-            "absolute_url": get_best_field_language(source["absolute_url"], language),
             "categories": source["categories"],
-            "cover_image": get_best_field_language(source["cover_image"], language),
-            "icon": get_best_field_language(source["icon"], language),
             "organization_highlighted": get_best_field_language(
                 source["organization_highlighted"], language
             )
@@ -601,7 +631,6 @@ class CoursesIndexer:
             else None,
             "organizations": source["organizations"],
             "state": CourseState(**state),
-            "title": get_best_field_language(source["title"], language),
         }
 
     @staticmethod
