@@ -91,7 +91,7 @@ class PersonsIndexer:
                 language: person.extended_object.get_absolute_url(language)
                 for language in titles.keys()
             },
-            "bio": {l: " ".join(st) for l, st in bio.items()},
+            "bio": {language: " ".join(st) for language, st in bio.items()},
             "complete": {
                 language: slice_string_for_completion(title)
                 for language, title in titles.items()
@@ -107,10 +107,14 @@ class PersonsIndexer:
         """
         index = index or cls.index_name
 
-        for person in Person.objects.filter(
-            extended_object__publisher_is_draft=False,
-            extended_object__title_set__published=True,
-        ).distinct():
+        for person in (
+            Person.objects.filter(
+                extended_object__publisher_is_draft=False,
+                extended_object__title_set__published=True,
+            )
+            .distinct()
+            .iterator()
+        ):
             yield cls.get_es_document_for_person(person, index=index, action=action)
 
     @staticmethod
